import os
import click
from pypiup.requirements import Requirements


BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


@click.command()
@click.option('--requirement', '-r', default='requirements.txt', type=click.STRING, help='Specify the path of the requirements file. Defaults to "requirements.txt".')
@click.option('--demo', '-d', is_flag=True, help='Load the demo requirements.txt file that comes with the package.')
def cli(requirement, demo):
    """
    PyPIup\n
    Check whether your PyPI requirements are up to date.
    """

    print("\n          ______   __  __     ______   __        __  __     ______  ")
    print("         /\  == \ /\ \_\ \   /\  == \ /\ \      /\ \/\ \   /\  == \ ")
    print("         \ \  _-/ \ \____ \  \ \  _-/ \ \ \     \ \ \_\ \  \ \  _-/ ")
    print("          \ \_\    \/\_____\  \ \_\    \ \_\     \ \_____\  \ \_\   ")
    print("           \/_/     \/_____/   \/_/     \/_/      \/_____/   \/_/   ")
    print("\n                                   PyPIup")
    print("                     https://github.com/manosim/pypiup")

    if demo:
        demo_path = os.path.join(BASE_DIR, 'requirements/requirements-demo.txt')
        return Requirements(demo_path)
    Requirements(requirement)

if __name__ == '__main__':
    cli()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import click
import re
import requests
import semantic_version


class Requirement(object):
    """
    Steps:
    1. Try to get name and version
    2. Get details from PyPI
    3. Compare Versions
    """

    def __init__(self, requirement):
        self.name = None
        self.current_version = None
        self.latest_version = None
        self.valid = False
        self.status = None
        self.setUp(requirement)

    def setUp(self, requirement):
        regex = re.split("==", requirement)
        try:
            self.name = regex[0]
            # Ignore inline comments if any
            self.current_version = regex[1].split("#")[0].rstrip()
            self.get_package_info()
            self.compare()
        except IndexError:
            # If it cannot find the name and version assume
            # it is not a pypi packages (ie. GitHub Repository)
            pass

    def get_package_info(self):
        pypi_url = "https://pypi.python.org/pypi/%s/json" % (self.name)
        request = requests.get(pypi_url)

        if request.status_code == 200:
            response_json = request.json()
            self.latest_version = response_json["info"]["version"]
            self.valid = True

    def valid_semver(self):
        valid_current = semantic_version.validate(self.current_version)
        valid_latest = semantic_version.validate(self.latest_version)
        return valid_current and valid_latest

    def compare(self):
        if self.valid_semver():
            comparison = semantic_version.compare(self.current_version, self.latest_version)
            if (comparison >= 0):
                self.status = "UPTODATE"
            else:
                self.status = "NEEDS_UPDATE"
        else:
            self.status = "INVALID_SEMVER"

    def output_details(self):
        if self.valid:
            if self.status == "UPTODATE":
                click.echo("\n%s\n  %s" % (click.style("✓ %s" % self.name, bold=True, fg='green'), click.style("Up to date, %s." % self.current_version)))
            elif self.status == "NEEDS_UPDATE":
                click.echo("\n%s\n  %s" % (click.style("✗ %s" % self.name, bold=True, fg='red'), click.style("Needs update, From %s to %s." % (self.current_version, self.latest_version))))
            elif self.status == "INVALID_SEMVER":
                click.echo("\n%s\n  %s" % (click.style("✗ %s" % self.name, bold=True, fg='cyan'), click.style("Could not compare. Invalid semver, From %s to %s." % (self.current_version, self.latest_version))))
        else:
            # Invalid Requirement (ie. GitHub Repository)
            pass



import unittest
import httpretty


class TestMockedRequests(unittest.TestCase):

    def setUp(self):
        super(TestMockedRequests, self).setUp()
        httpretty.enable()
        httpretty.register_uri(httpretty.GET, "https://pypi.python.org/pypi/Django/json", status=200, body='{"info": {"version": "1.9"}}', content_type="text/json")
        httpretty.register_uri(httpretty.GET, "https://pypi.python.org/pypi/djangorestframework/json", status=200, body='{"info": {"version": "3.3.2"}}', content_type="text/json")
        httpretty.register_uri(httpretty.GET, "https://pypi.python.org/pypi/drfdocs/json", status=200, body='{"info": {"version": "0.0.5"}}', content_type="text/json")
        httpretty.register_uri(httpretty.GET, "https://pypi.python.org/pypi/testing-errors-abc/json", status=404, body='Not found.', content_type="text/json")

    def tearDownModule(self):
        super(TestMockedRequests, self).tearDownModule()
        httpretty.disable()
        httpretty.reset()


class TestMockedDemoRequests(unittest.TestCase):

    def setUp(self):
        super(TestMockedDemoRequests, self).setUp()
        httpretty.enable()
        httpretty.register_uri(httpretty.GET, "https://pypi.python.org/pypi/Django/json", status=200, body='{"info": {"version": "1.9"}}', content_type="text/json")
        httpretty.register_uri(httpretty.GET, "https://pypi.python.org/pypi/djangorestframework/json", status=200, body='{"info": {"version": "3.3.1"}}', content_type="text/json")
        httpretty.register_uri(httpretty.GET, "https://pypi.python.org/pypi/django-crispy-forms/json", status=200, body='{"info": {"version": "1.5.0"}}', content_type="text/json")
        httpretty.register_uri(httpretty.GET, "https://pypi.python.org/pypi/click/json", status=200, body='{"info": {"version": "6.2"}}', content_type="text/json")
        httpretty.register_uri(httpretty.GET, "https://pypi.python.org/pypi/drfdocs/json", status=200, body='{"info": {"version": "0.0.5"}}', content_type="text/json")
        httpretty.register_uri(httpretty.GET, "https://pypi.python.org/pypi/apscheduler/json", status=200, body='{"info": {"version": "3.0.3"}}', content_type="text/json")
        httpretty.register_uri(httpretty.GET, "https://pypi.python.org/pypi/testing-errors-abc/json", status=404, body='Not found.', content_type="text/json")

    def tearDownModule(self):
        super(TestMockedDemoRequests, self).tearDownModule()
        httpretty.disable()
        httpretty.reset()



import unittest
from click.testing import CliRunner
from pypiup.cli import cli
from tests.mocks import TestMockedRequests, TestMockedDemoRequests


class TestRequirementsMockedRequests(TestMockedRequests):

    def setUp(self):
        super(TestRequirementsMockedRequests, self).setUp()

    def test_cli(self):
        runner = CliRunner()
        result = runner.invoke(cli, ['-r', 'requirements/requirements-tests.txt'])

        self.assertEqual(result.exit_code, 0)

        # Django
        self.assertIn("Could not compare. Invalid semver, From 1.7.1 to 1.9.", result.output)

        # djangorestframework
        self.assertIn("Up to date, 3.3.2.", result.output)

        # drfdocs
        self.assertIn("Needs update, From 0.0.4 to 0.0.5.", result.output)

        self.assertIn("Total Requirements: 5", result.output)
        self.assertIn("Up to date: 1", result.output)
        self.assertIn("Update available: 1", result.output)
        self.assertIn("Invalid Semver: 1", result.output)
        self.assertIn("Non PyPI Requirements: 2", result.output)


class TestRequirementsDemoMockedRequests(TestMockedDemoRequests):

    def setUp(self):
        super(TestRequirementsDemoMockedRequests, self).setUp()

    def test_cli(self):
        runner = CliRunner()
        result = runner.invoke(cli, ['--demo'])

        self.assertEqual(result.exit_code, 0)

        # Django
        self.assertIn("Could not compare. Invalid semver, From 1.7.1 to 1.9.", result.output)

        # click
        self.assertIn("Could not compare. Invalid semver, From 6.1 to 6.2.", result.output)

        # djangorestframework
        self.assertIn("Up to date, 3.3.2.", result.output)

        # django-crispy-forms
        self.assertIn("Needs update, From 1.4.0 to 1.5.0.", result.output)

        # drfdocs
        self.assertIn("Up to date, 0.0.5.", result.output)

        # apscheduler
        self.assertIn("Up to date, 3.0.3.", result.output)

        self.assertIn("Total Requirements: 8", result.output)
        self.assertIn("Up to date: 3", result.output)
        self.assertIn("Update available: 1", result.output)
        self.assertIn("Invalid Semver: 2", result.output)
        self.assertIn("Non PyPI Requirements: 2", result.output)


class TestRequirementsFileNotFound(unittest.TestCase):

    def test_cli_file_not_found(self):
        runner = CliRunner()
        result = runner.invoke(cli, ['-r', 'requirements-fake.txt'])

        self.assertEqual(result.exit_code, 0)
        self.assertIn("Could not find requirements-fake.txt. No such file or directory.", result.output)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import click
from pypiup.requirement import Requirement


class Requirements(object):

    def __init__(self, requirements_file):
        self.file = self.read_file(requirements_file)
        self.requirements = []

    def should_ignore_requirement(self, requirement):
        if requirement.startswith("#") or requirement == "":
            return True
        return False

    def read_file(self, requirements_filename):
        click.secho("\nAttempting to read %s ...\n" % requirements_filename, fg='magenta')

        try:
            requirements_file = open(requirements_filename)
            requirements_file_lines = requirements_file.read().splitlines()
            with click.progressbar(
                    iterable=requirements_file_lines,
                    label='Getting requirements details',
                    bar_template='%(label)s %(bar)s | %(info)s',
                    fill_char=click.style(u'█', fg='cyan'),
                    empty_char=' ') as bar:
                requirements = []
                for line in bar:
                    if not self.should_ignore_requirement(line):
                        requirement = Requirement(line)
                        requirements.append(requirement)
                self.requirements = requirements
                self.show_details()
                self.show_stats()
                requirements_file.close()
        except IOError:
            return click.secho("Could not find %s. No such file or directory.\n" % (requirements_filename), fg='red')

    def show_details(self):
        click.echo()
        for requirement in self.requirements:
            requirement.output_details()

    def show_stats(self):
        stat_total = len(self.requirements)
        stat_uptodate = 0
        stat_needs_update = 0
        stat_invalid_semver = 0
        stat_invalid = 0

        for requirement in self.requirements:
            if requirement.valid:
                if requirement.status == "UPTODATE":
                    stat_uptodate += 1
                elif requirement.status == "NEEDS_UPDATE":
                    stat_needs_update += 1
                elif requirement.status == "INVALID_SEMVER":
                    stat_invalid_semver += 1
            else:
                stat_invalid += 1

        click.echo(click.style("\nStatistics", bold=True, underline=True, fg='yellow'))
        click.echo("  Total Requirements: %s" % stat_total)
        click.echo("  Up to date: %s" % stat_uptodate)
        click.echo("  Update available: %s" % stat_needs_update)
        click.echo("  Invalid Semver: %s" % stat_invalid_semver)
        click.echo("  Non PyPI Requirements: %s" % stat_invalid)



from setuptools import find_packages, setup

setup(
    name='pypiup',
    version=__import__('pypiup').__version__,
    author="Emmanouil Konstantinidis",
    author_email="hello@manos.im",
    packages=find_packages(),
    include_package_data=True,
    url="https://github.com/manosim/pypiup",
    license='BSD',
    description="Check whether your PyPI requirements are up to date.",
    long_description=open("README.md").read(),
    install_requires=[
        'click',
        'requests',
        'semantic_version'
    ],
    entry_points='''
        [console_scripts]
        pypiup=pypiup.cli:cli
    ''',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ],
)
